==> visual-third-party-0.0.1-lg-sources.jar: com/tenqube/visual_third/manager/VisualAlarmManager.java <==
package com.tenqube.visual_third.manager;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.Build;

import com.tenqube.visual_third.Callback;
import com.tenqube.visual_third.entity.VisualNotification;
import com.tenqube.visual_third.receiver.AlarmReceiver;
import com.tenqube.visual_third.repository.RepositoryHolder;
import com.tenqube.visual_third.repository.ResourceRepository;
import com.tenqube.visual_third.repository.VisualRepository;
import com.tenqube.visual_third.util.Utils;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.TimeZone;

import static tenqube.parser.core.ParserService.mIsDebug;
import static tenqube.parser.util.LogUtil.LOGI;

public class VisualAlarmManager {

    public static final String TAG = VisualAlarmManager.class.getSimpleName();
    private static final int ALARM_SET = 0;
    private static final int ALARM_REPEAT = 1;

    private Context context;
    private static VisualAlarmManager mInstance = null;
    private AlarmManager alarmManager;
    private VisualRepository repository;

    private ResourceRepository resourceRepository;

    public static VisualAlarmManager getInstance(Context ctx) {
        synchronized (VisualAlarmManager.class) {
            if (mInstance == null) {
                mInstance = new VisualAlarmManager(ctx.getApplicationContext());
            }
        }
        return mInstance;
    }

    private VisualAlarmManager(Context context) {
        this.context = context;
        alarmManager = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
        repository = RepositoryHolder.getInstance(context).getVisualRepository();
        resourceRepository = RepositoryHolder.getInstance(context).getResourceRepository();
    }

    public void setReportTest(String type, int second) {

        boolean isAppNoti = repository.isAppNoti();

        ArrayList<VisualNotification> notifications = repository.loadNotifications();
        for(VisualNotification notification : notifications) {

            if(notification.getName().contains(type)) {
                cancelAlarm(notification);
                if(notification.isEnable() && isAppNoti) {
                    PendingIntent pIntent = createPendingIntent(notification);
                    long alarmTime = System.currentTimeMillis() + second * 1000;
                    notification.isTest = true;
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                        alarmManager.setExactAndAllowWhileIdle(AlarmManager.RTC_WAKEUP, alarmTime, pIntent);
                    } else {
                        alarmManager.set(AlarmManager.RTC_WAKEUP, alarmTime, pIntent);
                    }
                }
            }
        }

    }

    public void getNotification(int id, Callback<VisualNotification> callback) {
        VisualNotification result = repository.loadNotification(id);
        callback.onDataLoaded(result);
    }

    public void syncNotification() {
        resourceRepository.syncNotification();
    }

    /**
     * 호출 시점
     * 1. 처음 진입시
     * 2. 설정 변경시
     */
    public void setAlarms() {

        try {
            boolean isAppNoti = repository.isAppNoti();

            int minute = getMinute();
            ArrayList<VisualNotification> notifications = repository.loadNotifications();
            for(VisualNotification notification : notifications) {
                cancelAlarm(notification);
                if(notification.isEnable() && isAppNoti) {
                    setAlarm(notification, minute);
                }
            }
        } catch (Exception e) {

        }
    }

    private void setAlarm(VisualNotification notification, int minute) {

        PendingIntent pIntent = createPendingIntent(notification);
        long alarmTime = calculateAlarmTime(notification, minute);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            alarmManager.setExactAndAllowWhileIdle(AlarmManager.RTC_WAKEUP, alarmTime, pIntent);
        } else {
            alarmManager.set(AlarmManager.RTC_WAKEUP, alarmTime, pIntent);
        }
    }

    private void cancelAlarm(VisualNotification notification) {
        PendingIntent pIntent = createPendingIntent(notification);
        alarmManager.cancel(pIntent);
    }

    private PendingIntent createPendingIntent(VisualNotification notification) {
        Intent intent = new Intent(context, AlarmReceiver.class);
        intent.setAction(AlarmReceiver.ACTION_TENQUBE_NOTIFICATION);
        intent.putExtra(AlarmReceiver.ARG_NOTI_ID, notification.getId());
        intent.putExtra(AlarmReceiver.ARG_NOTI_TEST, notification.isTest);

        return PendingIntent.getBroadcast(context, notification.getId(), intent, PendingIntent.FLAG_UPDATE_CURRENT);
    }

    private long calculateAlarmTime(VisualNotification notification, int minute) {

        Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));

        long currentTime = calendar.getTimeInMillis();
        int currentDay = calendar.get(Calendar.DATE);

        int currentHour = calendar.get(Calendar.HOUR_OF_DAY);

        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);

        int day = notification.getDay();
        int dayOfWeek = notification.getDayOfWeek();
        int hour = notification.getHour();

        switch (notification.getName()) {
            case "monthly":
                if(day == currentDay) { // 1일로같음
                    if(currentHour >= hour) { // 알람설정 시간을 지나침
                        calendar.add(Calendar.MONTH, 1);
                    }
                } else {
                    calendar.add(Calendar.MONTH, 1);
                }

                calendar.set(Calendar.DATE, day);
                calendar.set(Calendar.HOUR_OF_DAY, hour);
                break;

            case "weekly_mon":
            case "weekly_fri":
                calendar.set(Calendar.HOUR_OF_DAY, hour);
                calendar.set(Calendar.DAY_OF_WEEK, dayOfWeek);

                if(currentTime >= calendar.getTimeInMillis()) {
                    calendar.add(Calendar.WEEK_OF_YEAR, 1);
                }
                break;

            case "daily":
                calendar.set(Calendar.HOUR_OF_DAY, hour);
//                calendar.set(Calendar.MINUTE, Calendar.getInstance().get(Calendar.MINUTE));
//                calendar.add(Calendar.MINUTE, 1);
                if(currentTime >= calendar.getTimeInMillis()) {
                    calendar.add(Calendar.DATE, 1);
                }
                break;
        }

        int beforeDay = calendar.get(Calendar.DATE);

        calendar.add(Calendar.MINUTE, minute);

        int afterDay = calendar.get(Calendar.DATE);

        if(beforeDay < afterDay) { // 하루가 지나가 버리면 다시 0으로 설정
            calendar.add(Calendar.MINUTE, -minute);
        }


        LOGI(TAG, "MINUTE: " + minute, mIsDebug);
        LOGI(TAG, "notification.getName()" + notification.getName(), mIsDebug);
        LOGI(TAG, "Alarm Date: " + Utils.getStringDateAsYYYYMMddHHmmss(calendar), mIsDebug);

        return calendar.getTimeInMillis();
    }

    /**
     * uid를 이용해 사용자 custId mod divider 를 통해 알람설정
     */
    private int getMinute() {

        int divider = repository.getDivider();
        String uid = repository.getUid();
        return Math.abs(uid.hashCode()) % divider;

    }

}

==> visual-third-party-0.4.4-lg-sources.jar: com/tenqube/visual_third/manager/VisualAlarmManager.java <==
package com.tenqube.visual_third.manager;

import android.content.Context;

import androidx.work.Data;
import androidx.work.ExistingWorkPolicy;
import androidx.work.OneTimeWorkRequest;
import androidx.work.WorkManager;

import com.tenqube.visual_third.entity.VisualNotification;
import com.tenqube.visual_third.repository.RepositoryHolder;
import com.tenqube.visual_third.repository.ResourceRepository;
import com.tenqube.visual_third.repository.VisualRepository;
import com.tenqube.visual_third.worker.ReportWorker;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;

public class VisualAlarmManager {

    public static final String TAG = VisualAlarmManager.class.getSimpleName();
    private static final int ALARM_SET = 0;
    private static final int ALARM_REPEAT = 1;

    private Context context;
    private static VisualAlarmManager mInstance = null;
    private VisualRepository repository;

    private ResourceRepository resourceRepository;

    public static VisualAlarmManager getInstance(Context ctx) {
        synchronized (VisualAlarmManager.class) {
            if (mInstance == null) {
                mInstance = new VisualAlarmManager(ctx.getApplicationContext());
            }
        }
        return mInstance;
    }

    private VisualAlarmManager(Context context) {
        this.context = context;
        repository = RepositoryHolder.getInstance(context).getVisualRepository();
        resourceRepository = RepositoryHolder.getInstance(context).getResourceRepository();
    }

    public void setReportTest(String type, int second) {

        boolean isAppNoti = repository.isAppNoti();

        ArrayList<VisualNotification> notifications = repository.loadNotifications();
        for(VisualNotification notification : notifications) {

            if(notification.getName().contains(type)) {

                if (notification.isEnable() && isAppNoti) {
                    notification.isTest = true;
                    startWorker(notification.getName(), notification.getId(), true, second * 1000);
                }
            }
        }
    }

    public VisualNotification getNotification(int id) {
        return repository.loadNotification(id);
    }

    public void syncNotification() {
        resourceRepository.syncNotification();
    }

    /**
     * 호출 시점
     * 1. 처음 진입시
     * 2. 설정 변경시
     */
    public void setAlarms() {

        try {
            boolean isAppNoti = repository.isAppNoti();

            int minute = getMinute();
            ArrayList<VisualNotification> notifications = repository.loadNotifications();

            for(VisualNotification notification : notifications) {
                if(notification.isEnable() && isAppNoti) {
                    long alarmTime = calculateAlarmTime(notification, minute);
                    startWorker(notification.getName(), notification.getId(), false, alarmTime);
                }
            }
        } catch (Exception e) {

        }
    }

    public void setAlarm(VisualNotification notification) {

        try {
            boolean isAppNoti = repository.isAppNoti();

            int minute = getMinute();

            if(notification.isEnable() && isAppNoti) {
                long alarmTime = calculateAlarmTime(notification, minute);
                startWorker(notification.getName(), notification.getId(), false, alarmTime);
            }

        } catch (Exception e) {

        }
    }

    private void startWorker(String reportName, int notiId, boolean isTest, long duration) {
        Data myData = new Data.Builder()
                // We need to pass three integers: X, Y, and Z
                .putString(ReportWorker.ARG_ACTION, ReportWorker.ACTION_TENQUBE_NOTIFICATION)
                .putInt(ReportWorker.ARG_NOTI_ID, notiId)
                .putBoolean(ReportWorker.ARG_NOTI_TEST, isTest)
                // ... and build the actual Data object:
                .build();

// ...then create and enqueue a OneTimeWorkRequest that uses those arguments
        OneTimeWorkRequest mathWork = new OneTimeWorkRequest.Builder(ReportWorker.class)
                .setInputData(myData)
                .addTag(reportName)
                .setInitialDelay(duration/ 1000, TimeUnit.SECONDS)
                .build();

        WorkManager.getInstance(context).enqueueUniqueWork(reportName, ExistingWorkPolicy.REPLACE, mathWork);
    }

    private long calculateAlarmTime(VisualNotification notification, int minute) {

        Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));

        long currentTime = calendar.getTimeInMillis();
        int currentDay = calendar.get(Calendar.DATE);

        int currentHour = calendar.get(Calendar.HOUR_OF_DAY);

        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);

        int day = notification.getDay();
        int dayOfWeek = notification.getDayOfWeek();
        int hour = notification.getHour();

        switch (notification.getName()) {
            case "monthly":
                if(day == currentDay) { // 1일로같음
                    if(currentHour >= hour) { // 알람설정 시간을 지나침
                        calendar.add(Calendar.MONTH, 1);
                    }
                } else {
                    calendar.add(Calendar.MONTH, 1);
                }

                calendar.set(Calendar.DATE, day);
                calendar.set(Calendar.HOUR_OF_DAY, hour);
                break;

            case "weekly_mon":
            case "weekly_fri":
                calendar.set(Calendar.HOUR_OF_DAY, hour);
                calendar.set(Calendar.DAY_OF_WEEK, dayOfWeek);

                if(currentTime >= calendar.getTimeInMillis()) {
                    calendar.add(Calendar.WEEK_OF_YEAR, 1);
                }
                break;

            case "daily":
                calendar.set(Calendar.HOUR_OF_DAY, hour);
//                calendar.set(Calendar.MINUTE, Calendar.getInstance().get(Calendar.MINUTE));
//                calendar.add(Calendar.MINUTE, 1);
                if(currentTime >= calendar.getTimeInMillis()) {
                    calendar.add(Calendar.DATE, 1);
                }
                break;
        }

        int beforeDay = calendar.get(Calendar.DATE);

        calendar.add(Calendar.MINUTE, minute);

        int afterDay = calendar.get(Calendar.DATE);

        if(beforeDay < afterDay) { // 하루가 지나가 버리면 다시 0으로 설정
            calendar.add(Calendar.MINUTE, -minute);
        }

        return calendar.getTimeInMillis() - currentTime;
    }

    /**
     * uid를 이용해 사용자 custId mod divider 를 통해 알람설정
     */
    private int getMinute() {

        int divider = repository.getDivider();
        String uid = repository.getUid();
        return Math.abs(uid.hashCode()) % divider;

    }

}
